==> scanbot_sdk-4.1.0-alpha01.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/camera/detectorWrappers/DocumentWrapper.kt <==
package io.scanbot.sdk.flutter.camera.detectorWrappers

import android.content.pm.PackageManager
import android.graphics.BitmapFactory
import android.graphics.Color
import android.util.Log
import androidx.camera.view.PreviewView
import com.google.gson.Gson
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodChannel
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.camera.CaptureInfo
import io.scanbot.sdk.camera.FrameHandlerResult
import io.scanbot.sdk.contourdetector.ContourDetectorFrameHandler
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewCallback
import io.scanbot.sdk.flutter.*
import io.scanbot.sdk.flutter.camera.CustomViewDisplayOwner
import io.scanbot.sdk.flutter.camera.config.DocumentScannerConfiguration
import io.scanbot.sdk.flutter.utils.convertDpToPx
import io.scanbot.sdk.persistence.PageFileStorage
import io.scanbot.sdk.persistence.PageImageSource
import io.scanbot.sdk.persistence.PageStorageProcessor
import io.scanbot.sdk.ui.camera.CameraUiSettings
import io.scanbot.sdk.ui.camera.ScanbotCameraContainerView
import io.scanbot.sdk.ui.camera.ScanbotCameraXView
import kotlinx.coroutines.*
import kotlin.math.max
import kotlin.math.min


class DocumentDetector(
    private val scannerView: DocumentScannerView,
    private val coroutineScope: CoroutineScope,
    private val cameraChannel: MethodChannel,
    private val detectorChannel: MethodChannel,
    override val scanbotSDK: ScanbotSDK,
    private val customViewDisplayOwner: CustomViewDisplayOwner,
    private val creationParams: HashMap<String, Any>,
    override val gson: Gson
) : IDetectorWrapper, CoroutineScope by coroutineScope {
    private val returnContext = Dispatchers.Main
    private val pageProcessor = scanbotSDK.createPageProcessor()
    private val pageFileStorage = scanbotSDK.createPageFileStorage()
    private val contourDetector = scanbotSDK.createContourDetector()
    private var shouldEnableDetection = true
    private var autoSnappingEnabled = true
    private var showPolygonInManualMode = false

    private var documentScannerConfig: DocumentScannerConfiguration = DocumentScannerConfiguration()
    private val handler = ContourDetectorFrameHandler.ResultHandler { result ->
        Log.d("CAMERA", "FRAME RESULT RETURNED")
        runBlocking {
            when (result) {
                is FrameHandlerResult.Success -> {
                    result.value.let { result ->
                        try {
                            scannerView.viewController.isFrameProcessingEnabled = false
                            val json = gson.toJsonTree(result).asJsonObject
                            withContext(returnContext) {
                                callMethodSync<Any>(detectorChannel, "onContourDetected", json)
                                scannerView.viewController.isFrameProcessingEnabled =
                                    shouldEnableDetection
                            }
                        } catch (e: Throwable) {
                            e.printStackTrace()
                        }
                    }
                }

                is FrameHandlerResult.Failure -> {
                    withContext(returnContext) {
                        val json = scanbotSDK.getLicenseStatusJson()
                        detectorChannel.invokeMethod(
                            "onLicenseError",
                            json.toString()
                        )
                    }
                }
            }
        }
        false
    }

    override fun attach(arguments: Json) {
        creationParams.apply {
            scannerView.initCamera(CameraUiSettings(true))
            val cameraXView =
                scannerView.findViewById<ScanbotCameraContainerView>(R.id.scanbot_camera_view)
            val previewView =
                cameraXView.findViewById<PreviewView>(io.scanbot.sdk.base.R.id.camera_preview_view)
            previewView.implementationMode = PreviewView.ImplementationMode.COMPATIBLE
            (cameraXView.getChildAt(0) as ScanbotCameraXView).lifecycleOwner =
                customViewDisplayOwner

            checkContourConfig()

            val configuration =
                (arguments).toDocumentScannerConfig(gson)
            documentScannerConfig = configuration
            addDetectorMethodHandler()
            scannerView.initDetectionBehavior(
                contourDetector,
                handler,
                object : IDocumentScannerViewCallback {
                    override fun onCameraOpen() {
                        creationParams.apply {
                            launch {
                                withContext(returnContext) {

                                    checkParam<Boolean>("flashEnabled")?.let {
                                        scannerView.viewController.useFlash(it)
                                    }
                                    checkParam<Double>("cameraZoomFactor")?.let {
                                        cameraXView.setPhysicalZoom(it.toFloat())
                                    }
                                    cameraChannel.invokeMethod("onCameraOpen", "")
                                }
                            }
                        }
                    }

                    override fun onPictureTaken(image: ByteArray, captureInfo: CaptureInfo) {
                        //todo possibly need to fix threading--

                        launch(Dispatchers.Default) {
                            saveSnappedImageAndReturn(image, captureInfo)
                            withContext(Dispatchers.Main) { scannerView.viewController.startPreview() }
                        }
                    }
                })
            checkScannerConfig(configuration)
            scannerView.viewController.isFrameProcessingEnabled = shouldEnableDetection
        }
    }

    private fun HashMap<String, Any>.checkContourConfig() {
        checkParam<Json>("contourConfiguration") {
            it.toContourConfig(gson).let {
               showPolygonInManualMode =  it.showPolygonInManualMode ?: showPolygonInManualMode

                it.autoSnapProgressEnabled?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnapProgressEnabled(it)
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColor(Color.parseColor(it))
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.fillOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColorOK(Color.parseColor(it))
                }
                it.strokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColor(Color.parseColor(it))
                }
                it.strokeOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColorOK(Color.parseColor(it))
                }
                it.autoSnapProgressStrokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.cornerRadius?.let {
                    scannerView.polygonConfiguration.setPolygonCornerRadius(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.autoSnapProgressStrokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.strokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
            }
        }
    }

    private fun checkScannerConfig(configuration: DocumentScannerConfiguration) {
        configuration.acceptedAngleScore?.let {
            val acceptedAngleScore = it.toDouble()
            if (acceptedAngleScore > 0 && acceptedAngleScore <= 100) {
                scannerView.viewController.setAcceptedAngleScore(acceptedAngleScore)
            }
        }
        configuration.acceptedSizeScore?.let {
            val acceptedSizeScore = it.toDouble()
            if (acceptedSizeScore > 0 && acceptedSizeScore <= 100) {
                scannerView.viewController.setAcceptedSizeScore(acceptedSizeScore)
            }
        }
        configuration.autoSnapSensitivity?.let {
            scannerView.viewController.setAutoSnappingSensitivity(it)
        }
        configuration.requiredAspectRatios?.let {
            scannerView.viewController.setRequiredAspectRatios(it)
        }
        configuration.ignoreBadAspectRatio?.let {
            scannerView.viewController.setIgnoreBadAspectRatio(it)
        }
        configuration.autoSnapEnabled?.let {
            autoSnappingEnabled = it
            scannerView.viewController.autoSnappingEnabled = it
            scannerView.polygonConfiguration.setPolygonViewVisible(autoSnappingEnabled || showPolygonInManualMode)
        }
    }

    private suspend fun saveSnappedImageAndReturn(image: ByteArray, captureInfo: CaptureInfo) {
        try {
            scannerView.viewController.autoSnappingEnabled = false
            scannerView.viewController.isFrameProcessingEnabled = false
            withContext(returnContext) { cameraChannel.invokeMethod("onShowHeavyProcessing", true) }
            val pageImageSource =
                if (captureInfo.isCapturedAutomatically) PageImageSource.AUTOSNAPPING else PageImageSource.MANUAL_SNAP

            val originalImageSizeLimit = documentScannerConfig.originalImageSizeLimit
                ?: PageStorageProcessor.Configuration.Size()
            val documentImageSizeLimit = documentScannerConfig.documentImageSizeLimit
                ?: PageStorageProcessor.Configuration.Size()

            val options: BitmapFactory.Options = BitmapFactory.Options()
            val originalBitmap = BitmapFactory.decodeByteArray(
                image,
                0,
                image.size,
                options
            )
            val maxConstraintSize =
                max(originalImageSizeLimit.width, originalImageSizeLimit.height)
            val maxBitmapSize = max(originalBitmap.width, originalBitmap.height)
            val downscale = min(1f, maxConstraintSize.toFloat() / maxBitmapSize.toFloat())

            val config = PageStorageProcessor.Configuration(
                imageOrientation = captureInfo.imageOrientation,
                imageScale = downscale,
                createDocument = documentScannerConfig.detectDocumentAfterSnap ?: false,
                pageImageSource = pageImageSource,
                documentImageSizeLimit = documentImageSizeLimit
            )

            val page = try {
                pageFileStorage.add(originalBitmap, config)
            } catch (error: OutOfMemoryError) {
                pageFileStorage.add(originalBitmap, config)
            }

            if (documentScannerConfig.detectDocumentAfterSnap != true) {
                val bitmap =
                    pageFileStorage.getImage(page.pageId, PageFileStorage.PageFileType.ORIGINAL)
                if (bitmap != null) {
                    pageFileStorage.setImageForId(
                        bitmap,
                        page.pageId,
                        PageFileStorage.PageFileType.UNFILTERED_DOCUMENT
                    )
                    pageFileStorage.setImageForId(
                        bitmap,
                        page.pageId,
                        PageFileStorage.PageFileType.DOCUMENT
                    )
                }
            }

            val pageJson = page.toJson(gson)
            insertPageUris(scanbotSDK, coroutineScope, page.pageId, pageJson as JsonObject)
            withContext(returnContext) {
                cameraChannel.invokeMethod("onShowHeavyProcessing", false)
                callMethodSync<Any>(cameraChannel, "onPictureTaken", pageJson)
                scannerView.viewController.autoSnappingEnabled = autoSnappingEnabled
                scannerView.viewController.isFrameProcessingEnabled = shouldEnableDetection
            }
        } catch (e: Throwable) {
            Log.e("ScanbotCameraModule", "error while saving snapped image", e)
        }
    }

    private fun addDetectorMethodHandler() {
        detectorChannel.setMethodCallHandler { call, result ->
            when (call.method) {
                "disable" -> {
                    shouldEnableDetection = false
                    scannerView.viewController.isFrameProcessingEnabled = false
                    result.success("success")
                }

                "enable" -> {
                    shouldEnableDetection = true
                    scannerView.viewController.isFrameProcessingEnabled = true
                    result.success("success")
                }

                "makeSnap" -> {
                    scannerView.viewController.takePicture(true)
                    result.success("success")
                }

                "setAutoSnappingEnabled" -> {
                    autoSnappingEnabled = call.argument<Boolean>("enabled") ?: false
                    documentScannerConfig.autoSnapEnabled = autoSnappingEnabled
                    scannerView.viewController.autoSnappingEnabled = autoSnappingEnabled
                    scannerView.polygonConfiguration.setPolygonViewVisible(autoSnappingEnabled || showPolygonInManualMode)
                    result.success("success")
                }
            }
        }
    }

    override fun detach() {
        detectorChannel.setMethodCallHandler(null)
    }

    override fun toggleContinuousFocus() {
        scannerView.viewController.continuousFocus()
    }

    override fun isFlashAvailable(): Boolean {
        return scannerView.context.packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH)
    }

    override fun flashEnabled(enabled: Boolean) {
        scannerView.viewController.useFlash(enabled)
    }
}

==> scanbot_sdk-4.2.1-alpha01.tar.gz: android/src/main/kotlin/io/scanbot/sdk/flutter/camera/detectorWrappers/DocumentWrapper.kt <==
package io.scanbot.sdk.flutter.camera.detectorWrappers

import android.content.pm.PackageManager
import android.graphics.BitmapFactory
import android.graphics.Color
import android.util.Log
import androidx.camera.view.PreviewView
import com.google.gson.Gson
import com.google.gson.JsonObject
import io.flutter.plugin.common.MethodChannel
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.camera.CaptureInfo
import io.scanbot.sdk.camera.FrameHandlerResult
import io.scanbot.sdk.contourdetector.ContourDetectorFrameHandler
import io.scanbot.sdk.docdetection.ui.DocumentScannerView
import io.scanbot.sdk.docdetection.ui.IDocumentScannerViewCallback
import io.scanbot.sdk.flutter.*
import io.scanbot.sdk.flutter.camera.CustomViewDisplayOwner
import io.scanbot.sdk.flutter.camera.config.DocumentScannerConfiguration
import io.scanbot.sdk.flutter.utils.convertDpToPx
import io.scanbot.sdk.persistence.PageFileStorage
import io.scanbot.sdk.persistence.PageImageSource
import io.scanbot.sdk.persistence.PageStorageProcessor
import io.scanbot.sdk.ui.camera.CameraUiSettings
import io.scanbot.sdk.ui.camera.ScanbotCameraContainerView
import io.scanbot.sdk.ui.camera.ScanbotCameraXView
import kotlinx.coroutines.*
import kotlin.math.max
import kotlin.math.min


class DocumentDetector(
    private val scannerView: DocumentScannerView,
    private val coroutineScope: CoroutineScope,
    private val cameraChannel: MethodChannel,
    private val detectorChannel: MethodChannel,
    override val scanbotSDK: ScanbotSDK,
    private val customViewDisplayOwner: CustomViewDisplayOwner,
    private val creationParams: HashMap<String, Any>,
    override val gson: Gson
) : IDetectorWrapper, CoroutineScope by coroutineScope {
    private val returnContext = Dispatchers.Main
    private val pageProcessor = scanbotSDK.createPageProcessor()
    private val pageFileStorage = scanbotSDK.createPageFileStorage()
    private val contourDetector = scanbotSDK.createContourDetector()
    private var shouldEnableDetection = true
    private var autoSnappingEnabled = true
    private var showPolygonInManualMode = false

    private var documentScannerConfig: DocumentScannerConfiguration = DocumentScannerConfiguration()
    private val handler = ContourDetectorFrameHandler.ResultHandler { result ->
        Log.d("CAMERA", "FRAME RESULT RETURNED")
        runBlocking {
            when (result) {
                is FrameHandlerResult.Success -> {
                    result.value.let { result ->
                        try {
                            scannerView.viewController.isFrameProcessingEnabled = false
                            val json = gson.toJsonTree(result).asJsonObject
                            withContext(returnContext) {
                                callMethodSync<Any>(detectorChannel, "onContourDetected", json)
                                scannerView.viewController.isFrameProcessingEnabled =
                                    shouldEnableDetection
                            }
                        } catch (e: Throwable) {
                            e.printStackTrace()
                        }
                    }
                }

                is FrameHandlerResult.Failure -> {
                    withContext(returnContext) {
                        val json = scanbotSDK.getLicenseStatusJson()
                        detectorChannel.invokeMethod(
                            "onLicenseError",
                            json.toString()
                        )
                    }
                }
            }
        }
        false
    }

    override fun attach(arguments: Json) {
        creationParams.apply {
            scannerView.viewController.setLifecycleOwner(customViewDisplayOwner)
            scannerView.initCamera(CameraUiSettings(true))
            val cameraXView =
                scannerView.findViewById<ScanbotCameraContainerView>(R.id.scanbot_camera_view)
            val previewView =
                cameraXView.findViewById<PreviewView>(io.scanbot.sdk.base.R.id.camera_preview_view)
            previewView.implementationMode = PreviewView.ImplementationMode.COMPATIBLE
            (cameraXView.getChildAt(0) as ScanbotCameraXView).lifecycleOwner =
                customViewDisplayOwner

            checkContourConfig()

            val configuration =
                (arguments).toDocumentScannerConfig(gson)
            documentScannerConfig = configuration
            addDetectorMethodHandler()
            scannerView.initDetectionBehavior(
                contourDetector,
                handler,
                object : IDocumentScannerViewCallback {
                    override fun onCameraOpen() {
                        creationParams.apply {
                            launch {
                                withContext(returnContext) {

                                    checkParam<Boolean>("flashEnabled")?.let {
                                        scannerView.viewController.useFlash(it)
                                    }
                                    checkParam<Double>("cameraZoomFactor")?.let {
                                        cameraXView.setPhysicalZoom(it.toFloat() * (12 - 1) + 1)
                                    }
                                    cameraChannel.invokeMethod("onCameraOpen", "")
                                }
                            }
                        }
                    }

                    override fun onPictureTaken(image: ByteArray, captureInfo: CaptureInfo) {
                        //todo possibly need to fix threading--

                        launch(Dispatchers.Default) {
                            saveSnappedImageAndReturn(image, captureInfo)
                            withContext(Dispatchers.Main) { scannerView.viewController.startPreview() }
                        }
                    }
                })
            checkScannerConfig(configuration)
            scannerView.viewController.isFrameProcessingEnabled = shouldEnableDetection
        }
    }

    private fun HashMap<String, Any>.checkContourConfig() {
        checkParam<Json>("contourConfiguration") {
            it.toContourConfig(gson).let {
               showPolygonInManualMode =  it.showPolygonInManualMode ?: showPolygonInManualMode

                it.autoSnapProgressEnabled?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnapProgressEnabled(it)
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColor(Color.parseColor(it))
                }
                it.fillColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.fillOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonFillColorOK(Color.parseColor(it))
                }
                it.strokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColor(Color.parseColor(it))
                }
                it.strokeOkColor?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeColorOK(Color.parseColor(it))
                }
                it.autoSnapProgressStrokeColor?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeColor(
                        Color.parseColor(
                            it
                        )
                    )
                }
                it.cornerRadius?.let {
                    scannerView.polygonConfiguration.setPolygonCornerRadius(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.autoSnapProgressStrokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonAutoSnappingProgressStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
                it.strokeWidth?.let {
                    scannerView.polygonConfiguration.setPolygonStrokeWidth(
                        convertDpToPx(
                            scannerView.context,
                            it
                        ).toInt()
                    )
                }
            }
        }
    }

    private fun checkScannerConfig(configuration: DocumentScannerConfiguration) {
        configuration.acceptedAngleScore?.let {
            val acceptedAngleScore = it.toDouble()
            if (acceptedAngleScore > 0 && acceptedAngleScore <= 100) {
                scannerView.viewController.setAcceptedAngleScore(acceptedAngleScore)
            }
        }
        configuration.acceptedSizeScore?.let {
            val acceptedSizeScore = it.toDouble()
            if (acceptedSizeScore > 0 && acceptedSizeScore <= 100) {
                scannerView.viewController.setAcceptedSizeScore(acceptedSizeScore)
            }
        }
        configuration.autoSnapSensitivity?.let {
            scannerView.viewController.setAutoSnappingSensitivity(it)
        }
        configuration.requiredAspectRatios?.let {
            scannerView.viewController.setRequiredAspectRatios(it)
        }
        configuration.ignoreBadAspectRatio?.let {
            scannerView.viewController.setIgnoreBadAspectRatio(it)
        }
        configuration.autoSnapEnabled?.let {
            autoSnappingEnabled = it
            scannerView.viewController.autoSnappingEnabled = it
            scannerView.polygonConfiguration.setPolygonViewVisible(autoSnappingEnabled || showPolygonInManualMode)
        }
    }

    private suspend fun saveSnappedImageAndReturn(image: ByteArray, captureInfo: CaptureInfo) {
        try {
            scannerView.viewController.autoSnappingEnabled = false
            scannerView.viewController.isFrameProcessingEnabled = false
            withContext(returnContext) { cameraChannel.invokeMethod("onShowHeavyProcessing", true) }
            val pageImageSource =
                if (captureInfo.isCapturedAutomatically) PageImageSource.AUTOSNAPPING else PageImageSource.MANUAL_SNAP

            val originalImageSizeLimit = documentScannerConfig.originalImageSizeLimit
                ?: PageStorageProcessor.Configuration.Size()
            val documentImageSizeLimit = documentScannerConfig.documentImageSizeLimit
                ?: PageStorageProcessor.Configuration.Size()

            val options: BitmapFactory.Options = BitmapFactory.Options()
            val originalBitmap = BitmapFactory.decodeByteArray(
                image,
                0,
                image.size,
                options
            )
            val maxConstraintSize =
                max(originalImageSizeLimit.width, originalImageSizeLimit.height)
            val maxBitmapSize = max(originalBitmap.width, originalBitmap.height)
            val downscale = min(1f, maxConstraintSize.toFloat() / maxBitmapSize.toFloat())

            val config = PageStorageProcessor.Configuration(
                imageOrientation = captureInfo.imageOrientation,
                imageScale = downscale,
                createDocument = documentScannerConfig.detectDocumentAfterSnap ?: false,
                pageImageSource = pageImageSource,
                documentImageSizeLimit = documentImageSizeLimit
            )

            val page = try {
                pageFileStorage.add(originalBitmap, config)
            } catch (error: OutOfMemoryError) {
                pageFileStorage.add(originalBitmap, config)
            }

            if (documentScannerConfig.detectDocumentAfterSnap != true) {
                val bitmap =
                    pageFileStorage.getImage(page.pageId, PageFileStorage.PageFileType.ORIGINAL)
                if (bitmap != null) {
                    pageFileStorage.setImageForId(
                        bitmap,
                        page.pageId,
                        PageFileStorage.PageFileType.UNFILTERED_DOCUMENT
                    )
                    pageFileStorage.setImageForId(
                        bitmap,
                        page.pageId,
                        PageFileStorage.PageFileType.DOCUMENT
                    )
                }
            }

            val pageJson = page.toJson(gson)
            insertPageUris(scanbotSDK, coroutineScope, page.pageId, pageJson as JsonObject)
            withContext(returnContext) {
                cameraChannel.invokeMethod("onShowHeavyProcessing", false)
                callMethodSync<Any>(cameraChannel, "onPictureTaken", pageJson)
                scannerView.viewController.autoSnappingEnabled = autoSnappingEnabled
                scannerView.viewController.isFrameProcessingEnabled = shouldEnableDetection
            }
        } catch (e: Throwable) {
            Log.e("ScanbotCameraModule", "error while saving snapped image", e)
        }
    }

    private fun addDetectorMethodHandler() {
        detectorChannel.setMethodCallHandler { call, result ->
            when (call.method) {
                "disable" -> {
                    shouldEnableDetection = false
                    scannerView.viewController.isFrameProcessingEnabled = false
                    result.success("success")
                }

                "enable" -> {
                    shouldEnableDetection = true
                    scannerView.viewController.isFrameProcessingEnabled = true
                    result.success("success")
                }

                "makeSnap" -> {
                    scannerView.viewController.takePicture(true)
                    result.success("success")
                }

                "setAutoSnappingEnabled" -> {
                    autoSnappingEnabled = call.argument<Boolean>("enabled") ?: false
                    documentScannerConfig.autoSnapEnabled = autoSnappingEnabled
                    scannerView.viewController.autoSnappingEnabled = autoSnappingEnabled
                    scannerView.polygonConfiguration.setPolygonViewVisible(autoSnappingEnabled || showPolygonInManualMode)
                    result.success("success")
                }
            }
        }
    }

    override fun detach() {
        detectorChannel.setMethodCallHandler(null)
    }

    override fun toggleContinuousFocus() {
        scannerView.viewController.continuousFocus()
    }

    override fun isFlashAvailable(): Boolean {
        return scannerView.context.packageManager.hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH)
    }

    override fun flashEnabled(enabled: Boolean) {
        scannerView.viewController.useFlash(enabled)
    }
}
